"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var executeMiddlewares = function executeMiddlewares(middlewares, webSocket, resolve) {
  if (middlewares.length === 0) {
    resolve({
      req: webSocket.upgradeReq
    });
  } else {
    var nextMiddleware = middlewares[0];
    var remainingMiddlewares = middlewares.slice(1);
    var response = {};
    nextMiddleware(webSocket.upgradeReq, response, function () {
      return executeMiddlewares(remainingMiddlewares, webSocket, resolve);
    });
  }
};
/**
 * Create the Apollo server subscriptions.onConnect option to use buildContext with subscriptions.
 * We need to apply the middlewares required for authentication to the websockets initial http
 * request. Usually, these middlewares are session(...), passport.initialize(...) and
 * passport.session(...). Provide the same instances of theses middlewares as used with your
 * express server.
 *
 * @param {array} middlewares session and passport related middlewares
 */


var createSubscriptionOnConnect = function createSubscriptionOnConnect(middlewares) {
  var onConnect = function onConnect(connectionParams, webSocket) {
    return new Promise(function (resolve) {
      return executeMiddlewares(middlewares, webSocket, resolve);
    });
  };

  return onConnect;
};

var _default = createSubscriptionOnConnect;
exports["default"] = _default;